Module compiler

Module compiler 

Parse, include expansion, component expansion, and binding pipeline. Defines bounded compiler contracts and behavior for this crate.

Structs

Compiler
Deterministic frontend compiler reusable across data instances.
CompilerBuilder
Builder for a compiler with explicit resolvers and limits.