pub enum Distribution {
Start,
Center,
End,
SpaceBetween,
SpaceAround,
SpaceEvenly,
}Expand description
Distribution of fixed-size children along a flow’s primary axis.
Variants
Start
Start at the leading edge and retain the declared gap.
Center
Center the complete flow.
End
Place the complete flow at the trailing edge.
SpaceBetween
Distribute remaining space only between children.
SpaceAround
Distribute remaining space around children using half edge spaces.
SpaceEvenly
Distribute equal space at edges and between children.