Typed deterministic data values and schemas. Defines bounded data contracts and behavior for this crate.
Structs
- Currency
Value - Exact monetary value.
- Data
Field - One schema field.
Enums
Functions
- resolve_
computed_ fields - Resolves computed schema fields through a bounded deterministic dependency graph.
- validate_
schema - Validates one object against a schema.
Type Aliases
- Data
Schema - Deterministically ordered schema.