pub struct ExportStyleOverride {
pub fill: Option<Color>,
pub stroke: Option<Color>,
pub opacity: Option<u32>,
pub color: Option<Color>,
}Expand description
Paint-only export layer applied after layout without changing geometry.
Fields
fill: Option<Color>Replacement fill for every resolved element and table cell.
stroke: Option<Color>Replacement stroke for every resolved element and table cell.
opacity: Option<u32>Replacement opacity in millionths.
color: Option<Color>Replacement text foreground.