pub struct Expression { /* private fields */ }Expand description
A parsed bounded expression without IO or arbitrary evaluation.
Implementations
Sourceimpl Expression
impl Expression
Sourcepub fn parse(source: impl Into<String>) -> Result<Self>
pub fn parse(source: impl Into<String>) -> Result<Self>
Parses a supported deterministic expression.
Sourcepub fn dependencies(&self) -> Vec<String>
pub fn dependencies(&self) -> Vec<String>
Returns root data fields referenced by the expression in lexical order.
This is used to build the computed-field dependency graph without evaluating the expression or consulting external state.