Module image

Module image 

Image crop, fit, focal point, EXIF, and resolved paint geometry. Defines bounded image contracts and behavior for this crate.

Structs

ImageCrop
Fractional crop insets expressed in parts per million.
ImageOptions
Source-independent image paint options.
ImagePlacement
Fully resolved image paint geometry consumed by exporters.
PixelRect
Integer source pixel rectangle.

Enums

ImageFit
How image pixels are mapped into an element box.
ImageOrientation
Orientation normalized from raster EXIF metadata.

Functions

resolve_image_placement
Reads bounded metadata and resolves crop/fit geometry before export.