pub struct DocumentIr {
pub template_id: String,
pub model: ModelKind,
pub page_size: Option<Size>,
pub page_template: Option<PageTemplate>,
pub collision: Option<CollisionPolicy>,
pub page_collision: Option<CollisionPolicy>,
pub guides: BTreeMap<String, Length>,
pub regions: BTreeMap<String, RegionIr>,
pub exclusions: BTreeMap<String, ExclusionIr>,
pub ai_policy: AiPolicy,
pub elements: Vec<ElementIr>,
}Expand description
Bound instance ready for measurement and layout.
Fields
template_id: StringTemplate identity.
model: ModelKindModel.
page_size: Option<Size>Optional page size.
page_template: Option<PageTemplate>Resolved page metadata.
collision: Option<CollisionPolicy>Bound document-wide collision policy.
page_collision: Option<CollisionPolicy>Bound page collision policy inherited after the document policy.
guides: BTreeMap<String, Length>Named guides.
regions: BTreeMap<String, RegionIr>Named regions.
exclusions: BTreeMap<String, ExclusionIr>Named page-relative exclusions.
ai_policy: AiPolicyOptional external-tool edit policy, retained but never executed by core.
elements: Vec<ElementIr>Bound root nodes.