pub struct LayoutTrace {
pub geometry: GeometryIr,
pub proposed: Rect,
pub collision_policy: CollisionPolicy,
pub initial_page: usize,
pub reflowed: bool,
}Expand description
Geometry and collision inputs retained for deterministic layout explanation.
Fields
geometry: GeometryIrPosition, size, constraints, region, and anchors from the bound IR.
proposed: RectProposed layout rectangle before collision or page reflow.
collision_policy: CollisionPolicyEffective inherited collision policy.
initial_page: usizePage proposed before pagination or collision reflow.
reflowed: boolWhether collision or pagination changed the proposed placement.