pub fn validate_data(
template: &TemplateIr,
data: &DataValue,
limits: &ResourceLimits,
) -> ValidationReportExpand description
Validates bounded typed data, its optional schema, and every binding.
pub fn validate_data(
template: &TemplateIr,
data: &DataValue,
limits: &ResourceLimits,
) -> ValidationReportValidates bounded typed data, its optional schema, and every binding.